package at.herold.test.objlifetime;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
/**
 * Thread to listen for admin events to start and stop monitoring.
 * @author Peter Kofler
 */
public class AdminThread extends Thread
{

   private static final int SERVER_PORT = 4444;

   public void run()
   {
      ServerSocket serverSocket;
      try
      {
         serverSocket = new ServerSocket(SERVER_PORT);
      }
      catch (IOException ex)
      {
         System.out.println("LTM error in Admin Thread");
         ex.printStackTrace();
         return;
      }

      while (true)
      {

         try
         {
            final Socket clientSocket = serverSocket.accept();

            final BufferedReader in = new BufferedReader(
               new InputStreamReader(clientSocket.getInputStream()));
            final String action = in.readLine();
            in.close();

            clientSocket.close();

            if ("start".equals(action))
            {
               ObjectLifetimeMonitoring.startMonitoring();
               System.out.println("LTM started");
            }
            else if ("stop".equals(action))
            {
               ObjectLifetimeMonitoring.stopMonitoring();
               System.out.println("LTM stopped");
            }
            else if ("save".equals(action))
            {
               ObjectLifetimeMonitoring.saveMonitorStatistics();
               System.out.println("LTM saved");
            }
            else if ("info".equals(action))
            {
               System.out.println("LTM - Number of monitored events: " + LifetimeStatistics.getNumberOfMonitoredActions());
               System.out.println("LTM - Number of time units since start: " + LifetimeStatistics.getNumberOfTimeUnitsSinceStart());
               System.out.println("LTM info");
            }
            else
            {
               System.out.println("LTM invalid command in Admin Thread");
            }

         }
         catch (IOException ex)
         {
            System.out.println("LTM error in Admin Thread");
            ex.printStackTrace();
         }

      }
   }

   public static void createAndStart()
   {
      // start admin listener
      final Thread admin = new AdminThread();
      admin.setDaemon(true);
      admin.start();
      System.out.println("LTM started Admin Thread");
   }

   public static void main(String[] args) throws IOException
   {
      if (args.length != 1)
      {
         System.out.println("sends Admin Command to Listener.");
         System.out.println("That are: start, stop, save, info.");
         return;
      }

      final Socket clientSocket = new Socket("127.0.0.1", SERVER_PORT);
      final PrintWriter out = new PrintWriter(clientSocket.getOutputStream(), true);
      out.println(args[0]);
      out.flush();
      out.close();
      clientSocket.close();
   }

}
