package at.herold.test.objlifetime.viewer;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
/**
 * Analyse the number of life objects per class. There is no graphically analyse.
 * @author Peter Kofler
 */
public class LifeObjectsPerClassAnalyser extends LifeTimeAnalyser
{
   
   /** List of classnames with numbers. */
   private final Map lifeObjectsPerClass = new HashMap();

   /** List of ClassWithNumber elements to sort. */
   final List numberList = new ArrayList();

   public void analyse(ViewedLifetimeStatistics pData)
   {
      data = pData;
      lifeObjectsPerClass.clear();

      countObjectsPerClass();

      sortNumberObjectsPerClass();

   }

   private void sortNumberObjectsPerClass()
   {
      for (Iterator i = lifeObjectsPerClass.keySet().iterator(); i.hasNext();)
      {
         final String key = (String) i.next();
         final Integer val = (Integer) lifeObjectsPerClass.get(key);

         ClassWithNumber cwn = new ClassWithNumber();
         cwn.name = key;
         cwn.number = val.intValue();
         numberList.add(cwn);
      }
      
      Collections.sort(numberList);
   }

   private void countObjectsPerClass()
   {
      final ViewedLifetime[] lives = data.getLifetimeArray();
      for (int i = 0; i < lives.length; i++)
      {
         final ViewedLifetime lt = lives[i];

         final String cn = lt.className;
         Integer numbers = (Integer) lifeObjectsPerClass.get(cn);
         if (numbers == null) {
            numbers = new Integer(0);
         }
         numbers = new Integer(numbers.intValue() + 1);
         lifeObjectsPerClass.put(cn, numbers);
      }
   }

   public void printResults()
   {
      System.out.println("-------------------- " +this.getClass().getName()+" --------------------");
      
      System.out.println(lifeObjectsPerClass.size() + " different classes used.");
      System.out.println(data.getNumberEvents() + " total number of classes used.");

      for (Iterator i = numberList.iterator(); i.hasNext();)
      {
         ClassWithNumber cwn = (ClassWithNumber) i.next();
         final int perc = (int) (10000.0/data.getNumberEvents()*cwn.number);
         System.out.println(cwn.number + " ; " + cwn.name + " ; " + (perc/100.0)+"%" );
      }
      
   }

   private static class ClassWithNumber implements Comparable
   {
      String name;
      int number;

      public int compareTo(Object o)
      {
         return ((ClassWithNumber) o).number - number;
      }
   }

}
