package at.herold.test.objlifetime.viewer;

import java.util.List;
/**
 * Viewed version of a creation or GC event. Two events are equal if they have
 * the same object identifier. But identifiers are not unique! They may come again
 * and again.
 * @author Peter Kofler
 */
abstract class ViewedEvent
{

   final int objectIdentifier;
   final int timeStamp;

   ViewedEvent(int pObjectIdentifier, int pTimeStamp)
   {
      objectIdentifier = pObjectIdentifier;
      timeStamp = pTimeStamp;
   }

   public String toString()
   {
      return objectIdentifier + " " + timeStamp;
   }

   public boolean equals(Object obj)
   {
      if (!(obj instanceof ViewedEvent))
         return false;

      return ((ViewedEvent) obj).objectIdentifier == objectIdentifier;
   }

   public int hashCode()
   {
      return objectIdentifier;
   }

   /**
    * Operate on the list which may or may not contain this or
    * the other C/D member for this.
    * @param creationEvents the list of creation events to remove from if any.
    * @param lifetimeEvents the list to put new lifetime events into.
    */
   public abstract void operateList(CreationEventList creationEvents, List lifetimeEvents);

}
