package at.herold.test.objlifetime.viewer;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
/**
 * Class for loading, preparing and wrapping the saved life time data.
 * This is also the central holder of statistics data.
 * @author Peter Kofler
 */
public class ViewedLifetimeStatistics
{

   private final CreationEventList creationEvents = new CreationEventListWithMap();
   private final List monitoredLifetimeList = new ArrayList();
   private ViewedLifetime[] monitoredLifetime;

   private int minTimeStamp;
   private int maxTimeStamp;
   private int numberLines;

   public int getMaxTimeStamp()
   {
      return maxTimeStamp;
   }

   public int getMinTimeStamp()
   {
      return minTimeStamp;
   }

   private void reset() {
      creationEvents.clear();
      monitoredLifetimeList.clear();
      minTimeStamp = Integer.MAX_VALUE;
      maxTimeStamp = 0;
      numberLines = 0;
   }

   public void load(String fileName) throws IOException
   {
      reset();

      final BufferedReader in = new BufferedReader(new FileReader(fileName));
      String t;
      while ((t = in.readLine()) != null)
      {
         numberLines++;
         ViewedEvent ev;

         final StringTokenizer tok = new StringTokenizer(t, " ");

         final String cmd = tok.nextToken();
         final String objectID = tok.nextToken().substring(1);
         final int time = Integer.parseInt(tok.nextToken());

         if (minTimeStamp>time) minTimeStamp = time;
         if (maxTimeStamp<time) maxTimeStamp = time;

         /*
         C @21997469 5333 java.nio.HeapCharBuffer
         D @2679386 5683
         E @ALL 10374
         */
         if (cmd.equals("C"))
         {
            ev = new ViewedCreationEvent(Integer.parseInt(objectID), time, tok.nextToken());

         }
         else if (cmd.equals("D"))
         {
            ev = new ViewedGCEvent(Integer.parseInt(objectID), time);

         }
         else if (cmd.equals("E"))
         {
            ev = new ViewedShutdownEvent(time);

         }
         else
         {
            throw new IOException("invalid data: " + t);
         }

         ev.operateList(creationEvents, monitoredLifetimeList);
      }

      in.close();

      if (creationEvents.size() > 0)
      {
         throw new IllegalArgumentException("no shutdown info in file " + fileName);
      }
   }

   public ViewedLifetime[] getLifetimeArray() {
      if (monitoredLifetime==null) {
         monitoredLifetime = (ViewedLifetime[]) monitoredLifetimeList.toArray(new ViewedLifetime[monitoredLifetimeList.size()]);
      }
      return monitoredLifetime;
   }

   public int getNumberLines()
   {
      return numberLines;
   }

   public int getNumberEvents() {
      return monitoredLifetimeList.size();
   }

}
